'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createClientResolver = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _reactRouter = require('react-router');

var _locationStorage = require('./locationStorage');

var _createResolver = require('./createResolver');

var createClientResolver = exports.createClientResolver = function createClientResolver(history, routes) {
  var custom = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];

  var resolver = (0, _createResolver.createResolver)();
  var locationStorage = (0, _locationStorage.createLocationStorage)();

  var transitionRule = function transitionRule(lastLocation, newLocation) {
    return lastLocation.pathname !== newLocation.pathname || lastLocation.search !== newLocation.search;
  };

  var setTransitionRule = function setTransitionRule(rule) {
    transitionRule = rule;
  };

  var isTransition = function isTransition(location) {
    return transitionRule(locationStorage.getLastLocation(), location);
  };

  var transition = function transition(location, continueTransition) {
    var forced = arguments.length <= 2 || arguments[2] === undefined ? false : arguments[2];

    if (!forced && !isTransition(location)) return;
    (0, _reactRouter.match)({ history: history, location: location, routes: routes }, function (error, redirectLocation, renderProps) {
      if (renderProps) {
        var attrs = _extends({}, custom, { location: location, params: renderProps.params });
        resolver.triggerHooks(renderProps.components, attrs, function () {
          continueTransition();
          locationStorage.setNewLocation(location);
        });
      }
    });
  };

  history.listenBefore(transition);

  var forceTrigger = function forceTrigger() {
    return transition(locationStorage.getLastLocation(), function () {}, true);
  };

  var setErrorHandler = function setErrorHandler(handler) {
    resolver.setErrorHandler(handler);
  };

  var addHooks = function addHooks() {
    resolver.addHooks.apply(resolver, arguments);
    return { addHooks: addHooks, setErrorHandler: setErrorHandler };
  };

  return {
    addHooks: addHooks,
    setErrorHandler: setErrorHandler,
    forceTrigger: forceTrigger,
    setTransitionRule: setTransitionRule
  };
};