'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createResolver = undefined;

var _createHooks = require('./createHooks');

var _helpers = require('./helpers');

var createResolver = exports.createResolver = function createResolver() {
  var registeredHooks = [];
  var errorHandler = function errorHandler() {};

  var setErrorHandler = function setErrorHandler(handler) {
    errorHandler = handler;
  };

  var triggerHooks = function triggerHooks(components, attrs) {
    var transition = arguments.length <= 2 || arguments[2] === undefined ? function () {} : arguments[2];

    var failed = false;
    var trigger = registeredHooks.reduce(function (promise, hookLayer) {
      return promise.then(function () {
        return Promise.all(hookLayer.filter(function (singleHook) {
          return !failed || singleHook.executeIfPreviousFailed;
        }).map(function (singleHook) {
          var hookAction = undefined;
          if ((0, _createHooks.isLocalHook)(singleHook)) hookAction = (0, _helpers.getDependencies)(components, singleHook.action, attrs);else if ((0, _createHooks.isGlobalHook)(singleHook)) hookAction = singleHook.action(attrs);else if ((0, _createHooks.isTransitionHook)(singleHook)) hookAction = transition();else {
            console.log('Invalid hook, skipping.', singleHook);
            hookAction = Promise.resolve();
          }
          if (!singleHook.stopOnException && hookAction && typeof hookAction.catch !== 'undefined') {
            hookAction = hookAction.catch(errorHandler);
          }
          return hookAction;
        })).catch(function (err) {
          failed = true;
          errorHandler(err);
        });
      });
    }, Promise.resolve());
    return trigger.then(function () {
      return failed ? Promise.reject() : Promise.resolve();
    });
  };

  var addHooks = function addHooks() {
    for (var _len = arguments.length, hooks = Array(_len), _key = 0; _key < _len; _key++) {
      hooks[_key] = arguments[_key];
    }

    registeredHooks.push(hooks.map(_createHooks.createHookObject));
    return { addHooks: addHooks, setErrorHandler: setErrorHandler };
  };

  return { addHooks: addHooks, setErrorHandler: setErrorHandler, triggerHooks: triggerHooks };
};